/*
TEST.PRG

This is a GAUSS program, written by
Bruce E. Hansen, Department of Economics, University of Wisconsin
bhansen@ssc.wisc.edu
http:/www.ssc.wisc.edu/~bhansen

This program tests the spline transfer function model presented in
"Are Private Transfers Altruistically Motivated?",
by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.

This program tests the spline model against polynomial alternatives.

The results create Table 4 from the paper.

*/

trm = .02;     @ Percentile of high-income households trimmed from sample@

qn   = 100;    @ number of quantiles used for test. We set qn=100 @
rep  = 1000;      @ Number of bootstrap replications.  @
_noinc=1;      @ If set to 1, includes dummy for no income @


output file = test.out reset;
output off;

/***********************************************************/


smp=1; do while smp<=2;     @ Set to 1 if "urban" or 2 if "rural"  @

let rname =
"Region1"
"Region2"
"Region3"
"Region4"
"Region5"
"Region6"
"Region7"
"Region8"
"Region9"
"Region10"
"Region11"
"Region12"
"Region13";
if smp==1;
  rname ="Region0"|rname;
  sample="Urban";
elseif smp==2;
  sample="Rural";
endif;

output on;
sh = sample $+ " households";
$sh;
"Trimmed Top Quantile of Income " trm;
"Bootstrap Replications         " rep;
"Number of Quantiles            " qn;
"Tests for a Single Knot";
"";
"     P       F-Test     P-Value";
output off;

dfile = "" $+ sample $+ ".dat";
open f1=^dfile;
dat=readr(f1,20000);
f1=close(f1);

abroad=dat[.,1];
domestic=dat[.,2];
inkind=dat[.,3];
gifts=dat[.,4];
totinc=dat[.,5];
retinc=dat[.,10];
yname = "Transfers";

y = abroad+domestic+inkind-gifts;
i = totinc+gifts-abroad-domestic-inkind-retinc;

dat=delif(dat~i~y,(i.<0));         @ delete observations with negative income @

dat = sortc(dat,28);
n = floor(rows(dat)*(1-trm));
dat = dat[1:n,.];
retinc=dat[.,10];
i=dat[.,28];
y=dat[.,29];
dat= dat[.,1:27];

dat[.,23]=dat[.,23]-dat[.,20]-dat[.,21]-dat[.,22];
xx=(retinc.>0)~retinc~dat[.,12:17];
region=dat[.,27];
r=unique(region,1);
xreg=(region.==(r'));
xx=xx~dat[.,18:26]~xreg;
mx = ones(n,1)*(meanc(xx)');
if _noinc==1;
  i0=(i.==0);
  xx=i0~xx;
endif;

p=1; do while p<=7;         @ Order of polynomial in splined sections @

/*if p<=4;  _per=.05;
elseif p<=6; _per=.15;
else; _per=.3;
endif;*/
_per=.05;
q = sortc(i,1);
q = q[floor(seqa(_per,(1-_per*2)/qn,qn)*n)];
{lr,e}=test(y,xx,i,q,p);


@ Bootstrap  @
boot = zeros(rep,1);
r=1; do while r<= rep;
  yb=e[ceil(rndu(n,1)*n)];
 {lrb,eb}=test(yb,xx,i,q,p);
  boot[r]=lrb;
r=r+1; endo;
pv = meanc(boot.>=lr);
output on;
p~lr~pv;
output off;

p=p+1;endo;
output on;
"";"";"";
output off;
smp = smp+1; endo;

/***********************************************************/

proc (2) = test(y,xx,i,q,p);
local sp,x,xm,e,ee0,sse,j,knot,dx,xv,beta,et,ssemin,lr;
  sp = seqa(1,1,p)';
  x = (i.^sp)~xx;
  xm = x*invpd(moment(x,0));
  e = y - xm*(x'y);
  ee0 = e'e;
  sse = zeros(qn,1);
  j=1;  do while j<=qn;
    knot = q[j];
    dx = ((i-knot).^sp).*(i.<=knot);
    xv = dx - xm*(x'dx);
    et = e - xv*div(e,xv);
    sse[j] = et'et;
  j=j+1; endo;
  j=minindc(sse);
  knot=q[j];
  ssemin = sse[j];
  lr = (ee0-ssemin)/ssemin*rows(y);
  dx = ((i-knot).^sp).*(i.<=knot);
  xv = dx - xm*(x'dx);
  et = e - xv*div(e,xv);
retp(lr,et);
endp;

proc div(e,x);
local beta;
    trap 1;
    beta=e/x;
    trap 0;
    if scalerr(beta);
      beta=pinv(moment(x,0))*(x'e);
    endif;
    if cols(beta)>1; beta=beta[.,1];endif;
retp(beta);
endp;

