/*
TEST2.PRG

This is a GAUSS program, written by
Bruce E. Hansen, Department of Economics, University of Wisconsin
bhansen@ssc.wisc.edu
http:/www.ssc.wisc.edu/~bhansen

This program tests the single-knot specification against a two-knot
alternative.

This results generate Table 5 from the paper.

*/

trm = .02;            @ Percentile of high-income households deleted  @
rep  = 1;          @ Number of bootstrap replications.  @

_per = .05;           @ Percentage to trim from quantiles for threshold search @
qn   = 1000;             @  Set qn to a positive integers to search
                          across a fixed number of quantiles     @
_noinc=1;             @ If set to 1, includes dummy for no income, 0 @

output file = test2.out reset;
output off;

/***********************************************************/

smp=1; do while smp<=2;              @ Set to 1 if "urban" or 2 if "rural"  @

let rname =
"Region1"
"Region2"
"Region3"
"Region4"
"Region5"
"Region6"
"Region7"
"Region8"
"Region9"
"Region10"
"Region11"
"Region12"
"Region13";
if smp==1;
  rname ="Region0"|rname;
  sample="Urban";
elseif smp==2;
  sample="Rural";
endif;

dfile = "" $+ sample $+ ".dat";
open f1=^dfile;
dat=readr(f1,20000);
f1=close(f1);

abroad=dat[.,1];
domestic=dat[.,2];
inkind=dat[.,3];
gifts=dat[.,4];
totinc=dat[.,5];
retinc=dat[.,10];
yname = "Transfers";

y = abroad+domestic+inkind-gifts;
i = totinc+gifts-abroad-domestic-inkind-retinc;

dat=delif(dat~i~y,(i.<0));         @ delete observations with negative income @

dat = sortc(dat,28);
n = floor(rows(dat)*(1-trm));
dat = dat[1:n,.];
retinc=dat[.,10];
i=dat[.,28];
y=dat[.,29];
dat= dat[.,1:27];

dat[.,23]=dat[.,23]-dat[.,20]-dat[.,21]-dat[.,22];
xx=(retinc.>0)~retinc~dat[.,12:17];
region=dat[.,27];
r=unique(region,1);
xreg=(region.==(r'));
xx=xx~dat[.,18:26]~xreg;
mx = ones(n,1)*(meanc(xx)');
if _noinc==1;
  i0=(i.==0);
  xx=i0~xx;
endif;

clear dat;
q = i[floor(seqa(_per,(1-_per*2)/qn,qn)*n)];
{f,yp,e} = test2(y,xx,i,q);

/*************************************************/
@ Bootstrap @

boot = zeros(rep,1);
r=1; do while r<= rep;
  yb=yp+e[ceil(rndu(n,1)*n)];
 {fb,ypb,eb}=test2(yb,xx,i,q);
  boot[r]=fb;
r=r+1; endo;
pv = meanc(boot.>=f);
output on;
sh = sample $+ " households";
$sh;
"Trimmed Top Quantile of Income " trm;
"Bootstrap Replications         " rep;
"Number of Quantiles            " qn;
"F Tests for Second Knot";
"";
"    F-Test     P-Value";
f~pv;
"";"";"";
output off;


/*************************************************/

smp = smp+1; endo;



/***********************************************************/

proc (3) = test2(y,xx,i,q);
local x,xm,e,ee0,sse,j1,j2,k1,dx,xv,beta,et,
sse0,i1,i2,i3,x1,x2,x3,k2,sse1,f,yp;
  xm = xx*invpd(moment(xx,0));
  e = y - xm*(xx'y);
  ee0 = e'e;
  sse = zeros(qn,1);
  j1=1;  do while j1<=qn;
    k1 = q[j1];
    i1 = (i.<k1);
    i2 = 1-i1;
    x1 = (i-k1).*i1;
    x2 = (i-k1).*i2;
    dx = x1~x2;
    xv = dx - xm*(xx'dx);
    et = e - xv*div(e,xv);
    sse[j1] = et'et;
  j1=j1+1; endo;
  j1=minindc(sse);
  k1=q[j1];
  sse0 = sse[j1];

  i1=(i.<k1);
  x1=i.*i1+(1-i1)*k1;
  sse = zeros(qn,1);
  j2=1;do while j2<=qn;
    if abs(j1-j2).>=ceil(_per*qn);
      k2 = q[j2];
      i2 = (i.<k2)-i1;
      i3 = 1-i1-i2;
      x2=(i-k1).*i2+i3*(k2-k1);
      x3=(i-k2).*i3;
      dx = x1~x2~x3;
      xv = dx - xm*(xx'dx);
      et = e - xv*div(e,xv);
      sse[j2] = et'et-sse0;
    endif;
  j2=j2+1;endo;
  j2=minindc(sse);
  k2=q[j2];
  sse1=sse[j2]+sse0;
  f=(sse0-sse1)/sse1*n;
  i2 = (i.<k2)-i1;
  i3 = 1-i1-i2;
  x2=(i-k1).*i2+i3*(k2-k1);
  x3=(i-k2).*i3;
  dx = x1~x2~x3;
  xv = dx - xm*(xx'dx);
  beta=div(e,xv);
  et = e - xv*beta;
  yp=dx*beta;
retp(f,yp,et);
endp;


proc div(e,x);
local beta;
    trap 1;
    beta=e/x;
    trap 0;
    if scalerr(beta);
      beta=pinv(moment(x,0))*(x'e);
    endif;
    if cols(beta)>1; beta=beta[.,1];endif;
retp(beta);
endp;

